@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2020 */
+/* Copyright (c) Philip Hazel, 1991 - 2021 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: November 2020 */
+/* This file last modified: January 2021 */
 
 
 /* This file contains all the global data */
@@ -148,6 +148,7 @@ BOOL main_transposedaccforce;
 trkeystr *main_transposedkeys;
 int  main_truepagelength;
 uschar main_xkeys[MAX_XKEYS][10];
+uschar main_xkeyorder[MAX_XKEYS][10];
 
 uschar *midi_filename = NULL;
 BOOL    midi_for_notes_off = FALSE;
