@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2020 */
+/* Copyright (c) Philip Hazel, 1991 - 2021 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: April 2020 */
+/* This file last modified: January 2021 */
 
 
 /* This file contains miscellaneous subroutines for outputting a page */
@@ -321,7 +321,9 @@ if (key63 < X_key)
     }
   }
 
-/* Handle custom key signature */
+/* Handle custom key signature. To preserve the accidental order in which the
+key was defined, we use an additional vector, which can be terminated early by
+a value greater than 9. */
 
 else
   {
@@ -330,14 +332,14 @@ else
 
   for (i = 0; i < 10; i++)
     {
-    int level, offset;
-    int baseacc = p[i] & 0x7f;
+    int level, offset, baseacc;
+    int k = main_xkeyorder[key63 - X_key][i];
 
-    if (baseacc == ac_none) continue;
+    if (k > 9) break;       /* Early end of order list (normally the case) */
+    baseacc = p[k] & 0x7f;
 
-    level = i + clef_offset;
-    if (level > 9) level -= 7;
-      else if (level < 0) level += 7;
+    level = k + clef_offset;
+    if (level > 9) level -= 7; else if (level < 0) level += 7;
 
     if (use_naturals)
       {
@@ -346,7 +348,7 @@ else
     else
       {
       offset = baseacc;
-      if ((p[i] & 0x80) != 0)
+      if ((p[k] & 0x80) != 0)
         offset += ((offset == ac_sharp)? (curmovt->hsharpstyle == 0) :
           (curmovt->hflatstyle == 0))? 18:36;
       }
