@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2020 */
+/* Copyright (c) Philip Hazel, 1991 - 2021 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: November 2020 */
+/* This file last modified: January 2021 */
 
 /* This is the main header file, imported by all other sources. */
 
@@ -1295,6 +1295,7 @@ extern BOOL main_transposedaccforce;   /* TRUE => retain accs when transposed */
 extern trkeystr *main_transposedkeys;  /* Transposed keys masquerades */
 extern int  main_truepagelength;       /* Actual page length */
 extern uschar main_xkeys[MAX_XKEYS][10]; /* Custom key signatures */
+extern uschar main_xkeyorder[MAX_XKEYS][10]; /* Acc order for custom keys */
 
 extern uschar *midi_filename;          /* name of MIDI file */
 extern BOOL    midi_for_notes_off;     /* If TRUE, do MIDI even if [notes off] */
