@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 2nd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2020 */
+/* Copyright (c) Philip Hazel, 1991 - 2021 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: December 2020 */
+/* This file last modified: January 2021 */
 
 
 /* This file contains part I of the code for reading in a PMW score file. */
@@ -1331,6 +1331,7 @@ memcpy(main_notespacing, init_notespacing, 8*sizeof(int));
 /* All custom keys default to no key signature. */
 
 memset(main_xkeys, 0, sizeof(main_xkeys));
+memset(main_xkeyorder, 255, sizeof(main_xkeyorder));
 
 /* Initialize movement table for first movement */
 
