@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2020 */
+/* Copyright (c) Philip Hazel, 1991 - 2021 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: November 2020 */
+/* This file last modified: January 2021*/
 
 
 /* This file contains part II of the code for reading in a PMW score file. It
@@ -1274,12 +1274,14 @@ curmovt->ledger = (curmovt->ledger == 0)? '=' : 184;
 *             Makekey                            *
 *************************************************/
 
-/* Read the definition of a custom key signature. */
+/* Read the definition of a custom key signature. We preserve the accidentals
+specified for each line or space, and also the order in which they are defined.
+A maximum of 10 are supported. */
 
 static void
 makekey(void)
 {
-int i, n;
+int i, j, n;
 
 sigch();
 if (toupper(read_ch) != 'X' ||
@@ -1293,7 +1295,7 @@ if (toupper(read_ch) != 'X' ||
 n--;  /* Now in range 0-9 */
 for (i = 0; i < 10; i++) main_xkeys[n][i] = 0;  /* Reset all accidentals */
 
-for (;;)
+for (j = 0; j < 10; j++)
   {
   int ac;
 
@@ -1342,6 +1344,7 @@ for (;;)
     break;
 
     default:
+    if (j < 10) main_xkeyorder[n][j] = 255;  /* Mark end of order list */
     return;
     }
 
@@ -1353,6 +1356,7 @@ for (;;)
     }
 
   main_xkeys[n][i] = ac;
+  main_xkeyorder[n][j] = i;
   }
 }
 
