@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2016 */
+/* Copyright (c) Philip Hazel, 1991 - 2020 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: June 2016 */
+/* This file last modified: April 2020 */
 
 
 /* This file contains code for miscellanous functions used
@@ -116,9 +116,9 @@ the curve is the same as the x coordinate of the point that is the fraction of
 the x-distance between the end points. This is used when drawing partial
 curves.
 
-The code in this function had to be massaged carefully to ensure that exactly 
-the same result is obtained when run native and under valgrind, in order to 
-make the tests run clean. The differences were only in the 3rd decimal place, 
+The code in this function had to be massaged carefully to ensure that exactly
+the same result is obtained when run native and under valgrind, in order to
+make the tests run clean. The differences were only in the 3rd decimal place,
 insignificant in the actual output, but of course the comparisons failed.
 
 Arguments:
@@ -148,11 +148,11 @@ for (;;)
   for (t = start; t < stop + step; t += step)
     {
     double x = ((a*t + b)*t + c)*t + x0;
-    
-    /* Make sure that -0.0 is actually 0.0; this can be different when running 
+
+    /* Make sure that -0.0 is actually 0.0; this can be different when running
     valgrind. */
-    
-    if (fabs(x) < 0.000001) x = 0.0; 
+
+    if (fabs(x) < 0.000001) x = 0.0;
 
     /* If stepped past the wanted point, set new bounds and a smaller step
     unless we are close enough. */
@@ -376,6 +376,8 @@ int lineslur = (slurflags & sflag_l) != 0;
 int below = (slurflags & sflag_b) != 0;
 int absolute = (slurflags & sflag_abs) != 0;
 int lay = (slurflags & sflag_lay) != 0;
+int use_endmoff = ((slurflags & sflag_cx) != 0)? out_moff : out_lastmoff;
+int line_rc_adjust = lineslur? 3000 : 0;
 
 int x0 = s->x;
 int y0 = s->y;
@@ -476,7 +478,11 @@ else if (below)    /* slur below */
   {
   int miny = s->miny;
 
-  if (!laststemup && (n_flags & nf_stem) != 0 && !lineslur && !eol)
+  if (!laststemup &&               /* Note has down stem */
+      (n_flags & nf_stem) != 0 &&  /* Note has a stem */
+      !lineslur &&                 /* Not a line (i.e. a slur) */
+      !eol &&                      /* Not an end of line slur */
+      !main_righttoleft)           /* Not right-to-left */
     x1 -= 5*main_stavemagn;
 
   if (s->count > 3)
@@ -616,6 +622,20 @@ if (sol || eol)
   while (sm != NULL && sm->sequence != s->section) sm = sm->next;
   if (sm != NULL)
     {
+    if (sm->lxoffset != 0)
+      {
+      int offset = mac_muldiv(len_crotchet, sm->lxoffset, 1000);
+      x0 = out_barx + out_findAoffset(s->moff + offset);
+      }
+
+    /* When x1 default for a line is set by musical offset, lose the additional
+    3pts that are added to get past the final note in the default case. */
+
+    if (sm->rxoffset != 0)
+      {
+      int offset = mac_muldiv(len_crotchet, sm->rxoffset, 1000);
+      x1 = out_barx + out_findAoffset(use_endmoff + offset) - line_rc_adjust;
+      }
     x0 += sm->lx;
     x1 += sm->rx;
     y0 += sm->ly;
@@ -645,6 +665,11 @@ else if (sol)
   {
   if (smm != NULL)
     {
+    if (smm->rxoffset != 0)
+      {
+      int offset = mac_muldiv(len_crotchet, smm->rxoffset, 1000);
+      x1 = out_barx + out_findAoffset(use_endmoff + offset) - line_rc_adjust;
+      }
     x1 += smm->rx;
     y1 += smm->ry;
     y0 += smm->ry;
@@ -659,6 +684,11 @@ else if (eol)
   {
   if (smm != NULL)
     {
+    if (smm->lxoffset != 0)   /* Relative to start of slur */
+      {
+      int offset = mac_muldiv(len_crotchet, smm->lxoffset, 1000);
+      x0 = out_barx + out_findAoffset(s->moff + offset);
+      }
     x0 += smm->lx;
     y0 += smm->ly;
     y1 += smm->ly;
@@ -669,6 +699,16 @@ else if (eol)
 
 else if (smm != NULL)
   {
+  if (smm->lxoffset != 0)  /* Relative to start of slur */
+    {
+    int offset = mac_muldiv(len_crotchet, smm->lxoffset, 1000);
+    x0 = out_barx + out_findAoffset(s->moff + offset);
+    }
+  if (smm->rxoffset != 0)
+    {
+    int offset = mac_muldiv(len_crotchet, smm->rxoffset, 1000);
+    x1 = out_barx + out_findAoffset(use_endmoff + offset) - line_rc_adjust;
+    }
   x0 += smm->lx;
   x1 += smm->rx;
   y0 += smm->ly;
@@ -814,10 +854,10 @@ if (lineslur && s->gaps != NULL)
     in the gap. */
 
     if (lg->gaptext != NULL)
-      { 
+      {
       out_gaptext(lg->gaptext, (xg0 + x0)/2,
         (y0 + yg0)/2 + (below? (-3000) : (3000)), num, den);
-      }   
+      }
 
     /* Extract from the chain and free the block */
 
@@ -933,7 +973,7 @@ else
   {
   if (x0 >= x1)
     {
-    error_moan(63, out_bar, out_stave);
+    error_moan(ERR63, out_bar, out_stave);
     if (x0 == x1) x1 = x0 + 10000; else
       { int temp = x0; x0 = x1; x1 = temp; }
     }
@@ -1082,13 +1122,13 @@ if (v > 10000.0) v = 10000.0;
 
 /* It is necessary to use floor() in the conversion of xx*0.3 to an integer in
 the next statement in order to get the same value under valgrind. We know that
-xx is positive, so we don't need to test whether to use floor() or ceil(). 
-Using the (int) cast only on a variable (not on a function) avoids a compiler 
+xx is positive, so we don't need to test whether to use floor() or ceil().
+Using the (int) cast only on a variable (not on a function) avoids a compiler
 warning. */
 
 temp = floor(xx * 0.3);
 co = (above * (co + ((xx > 20000)? 6000 : (int)temp)) * main_stavemagn)/1000;
-  
+
 f = ((double)start)/1000.0;
 g = ((double)stop)/1000.0;
 
@@ -1127,10 +1167,10 @@ if (f > 0.0 && f < 1.0) f = bezfraction(f, ax, bx, cx, x0, x3);
 if (g > 0.0 && g < 1.0) g = bezfraction(g, ax, bx, cx, x0, x3);
 
 /* Now calculate the new Bezier point coordinates for the portion of the slur
-that we want, and set up the first path to be that portion. We used to compute 
-the x values with just an (int) cast, but this gave slightly different values 
-under valgrind. Using floor() or ceil() with a rounding value solves that 
-problem. We must also avoid using an (int) cast directly on these functions, 
+that we want, and set up the first path to be that portion. We used to compute
+the x values with just an (int) cast, but this gave slightly different values
+under valgrind. Using floor() or ceil() with a rounding value solves that
+problem. We must also avoid using an (int) cast directly on these functions,
 because it provokes a compiler warning when -Wbad-function-cast is set. */
 
 zz[0] = x0 + ((ax*f + bx)*f + cx)*f;
